@@ -1,8 +1,10 @@
+from functools import singledispatch
 from importlib import import_module
 from pathlib import Path
+from typing import Any
 from typing import Callable
 from typing import Optional
-from typing import Tuple
+from typing import Sequence
 
 import jsonschema
 import yaml
@@ -11,42 +13,48 @@ from flask_socketio import SocketIO
 
 from asynction.types import GLOBAL_NAMESPACE
 from asynction.types import AsyncApiSpec
+from asynction.types import ChannelBindings
 from asynction.types import ChannelHandlers
 from asynction.types import JSONMapping
-from asynction.types import JSONMappingValue
+from asynction.validation import bindings_validator_factory
 from asynction.validation import payload_validator_factory
 from asynction.validation import validate_payload
 
 
+@singledispatch
+def deep_resolve(unresovled: Any, resolver: jsonschema.RefResolver) -> Any:
+    return unresovled
+
+
+@deep_resolve.register(dict)
+def _deep_resolve_mapping(
+    unresolved: JSONMapping, resolver: jsonschema.RefResolver
+) -> JSONMapping:
+    return {
+        k: resolver.resolve(v["$ref"])[-1] if "$ref" in v else deep_resolve(v, resolver)
+        for k, v in unresolved.items()
+    }
+
+
+@deep_resolve.register(list)
+@deep_resolve.register(tuple)
+@deep_resolve.register(set)
+def _deep_resolve_sequence(
+    unresolved: Sequence, resolver: jsonschema.RefResolver
+) -> Sequence:
+    return unresolved.__class__(  # type: ignore
+        [
+            resolver.resolve(item["$ref"])[-1]
+            if "$ref" in item
+            else deep_resolve(item, resolver)
+            for item in unresolved
+        ]
+    )
+
+
 def resolve_references(raw_spec: JSONMapping) -> JSONMapping:
     resolver = jsonschema.RefResolver.from_schema(raw_spec)
-
-    def deep_resolve(unresolved: JSONMapping) -> JSONMapping:
-        def transform(
-            item: Tuple[str, JSONMappingValue]
-        ) -> Tuple[str, JSONMappingValue]:
-            k, v = item
-            if isinstance(v, dict):
-                resolved_v = (
-                    resolver.resolve(v["$ref"])[-1] if "$ref" in v else deep_resolve(v)
-                )
-                return k, resolved_v
-            if isinstance(v, (list, tuple, set)):
-                resolved_v = v.__class__(
-                    [
-                        resolver.resolve(item["$ref"])[-1]
-                        if "$ref" in item
-                        else deep_resolve(item)
-                        for item in v
-                    ]
-                )
-                return k, resolved_v
-            else:
-                return k, v
-
-        return dict(map(transform, unresolved.items()))
-
-    return deep_resolve(raw_spec)
+    return deep_resolve(raw_spec, resolver)
 
 
 def load_spec(spec_path: Path) -> AsyncApiSpec:
@@ -92,10 +100,18 @@ class AsynctionSocketIO(SocketIO):
         return asio
 
     def _register_namespace_handlers(
-        self, namespace: str, channel_handlers: ChannelHandlers
+        self,
+        namespace: str,
+        channel_handlers: ChannelHandlers,
+        channel_bindings: Optional[ChannelBindings],
     ) -> None:
         if channel_handlers.connect is not None:
             handler = load_handler(channel_handlers.connect)
+
+            if self.validation:
+                with_bindings_validation = bindings_validator_factory(channel_bindings)
+                handler = with_bindings_validation(handler)
+
             self.on_event("connect", handler, namespace)
 
         if channel_handlers.disconnect is not None:
@@ -125,7 +141,9 @@ class AsynctionSocketIO(SocketIO):
                     self.on_event(message.name, handler, namespace)
 
             if channel.x_handlers is not None:
-                self._register_namespace_handlers(namespace, channel.x_handlers)
+                self._register_namespace_handlers(
+                    namespace, channel.x_handlers, channel.bindings
+                )
 
     def emit(self, event: str, *args, **kwargs) -> None:
         if self.validation:
